#!/bin/bash
#
# Copyright (c) 2019-2020 Oracle and/or its affiliates. All rights reserved.
# Licensed under the GNU General Public License Version 2 as shown at https://oss.oracle.com/licenses/GPL-2

set -o nounset
set -o pipefail
set -o errtrace

# variables
TO_REGISTRY=
FROM_REGISTRY=container-registry.oracle.com/olcne
MODULE_DIRECTORY=/etc/olcne/modules
USELOCAL=0
MODULE_VERSION=
MODULE_NAME=

function exit_trap {
    local rc=$?
    local lc="$BASH_COMMAND"

    if [[ $rc -ne 0 ]]; then
        echo "Command [$lc] exited with code [$rc]"
    fi
}

trap exit_trap EXIT

function pull {
    image=$1

    echo ">> Pulling image: ${FROM_REGISTRY}/${image}"
    podman pull "${FROM_REGISTRY}/${image}"
}

function tag {
    image=$1

    echo ">> Tagging image: ${TO_REGISTRY}/${image}"
    podman tag "${FROM_REGISTRY}/${image}" "${TO_REGISTRY}/${image}"
}

function push {
    image=$1

    echo ">> Pushing image: ${TO_REGISTRY}/${image}"
    podman push "${TO_REGISTRY}/${image}"
}

function getKubernetesImages {
    kubernetesVersion=$1

    # 104 is hardcoded to the exact lines the version field is in the manifest
    grep -A104 'versions:' ${MODULE_DIRECTORY}/kubernetes/1.0.0/kubernetes.yaml | yq r - versions.["${kubernetesVersion}"].container-images
}

function getIstioImages {
    istioVersion=$1

    # 60 is hardcoded to the exact lines the version field is in the manifest
    grep -A60 'versions:' ${MODULE_DIRECTORY}/istio/1.0.0/istio.yaml | yq r - versions.["${istioVersion}"].container-images
}

function getPrometheusImages {
    prometheusVersion=$1

    # 3 is hardcoded to the exact lines the version field is in the manifest
    grep -A3 'versions:' ${MODULE_DIRECTORY}/prometheus/1.0.0/prometheus.yaml | yq r - versions.["${prometheusVersion}"].container-images
}

function getImages {
    all_images_string="*"

    module="${1:-ALL}"
    moduleVersion="${2:-${all_images_string}}"

    case ${module} in
        kubernetes)
            getKubernetesImages "${moduleVersion}"
            ;;
        istio)
            getIstioImages "${moduleVersion}"
            ;;
        prometheus)
            getPrometheusImages "${moduleVersion}"
            ;;
        ALL)
            getKubernetesImages "${moduleVersion}"
            getIstioImages "${moduleVersion}"
            getPrometheusImages "${moduleVersion}"
            ;;
    esac
}

function check {
    local from="${1:-${REGISTRY}}"
    local to="${2:-}"
    local test_image="pause:3.1"

    # start docker-engine
    echo "Checking if podman is installed ..."
    if ! podman --help > /dev/null; then
        echo "[ERROR] podman is not install ... please install podman"
        exit 1
    fi

    echo "Checking if yq is installed ..."
    if ! yq --help > /dev/null; then
        echo "[ERROR] yq is not install ... please install yq"
        exit 1
    fi
}

function usage {
    echo "This script is to help pulling container images from default container-registry.oracle.com to a local repo" >&2
    echo "usage: " >&2
    echo "  $0 --to registry [--local --from registry --module <MODULE NAME> --version <MODULE VERSION>]" >&2
    exit 1
}

function main {
    # Load all local images
    if [ "$USELOCAL" -eq 1 ]; then
        if ls /usr/local/share/kubeadm/*.tar &> /dev/null; then
            find /usr/local/share/kubeadm/*.tar -print0 | xargs -0 | xargs -n1 podman load -i
        fi
        if ls /usr/local/share/olcne/*.tar &> /dev/null; then
            find /usr/local/share/olcne/*.tar -print0 | xargs -0 | xargs -n1 podman load -i
        fi
    fi

    # echo $imageInfo
    images=$(getImages "${MODULE_NAME}" "${MODULE_VERSION}")

    # Loop through all the images
    while IFS= read -r imageInfo ; do
        # Remove space to from the image name e.g. 'proxyv2: 1.7.3'
        image="${imageInfo// /}"
        success=false
        for i in {1..3}; do
            # Only pull the image if we are not looking at local images
            if [ "$USELOCAL" -eq 0 ]; then
                pull "$image"
                if [[ $? -ne 0 ]]; then
                    continue
                fi
            fi

            tag "$image"
            if [[ $? -ne 0 ]]; then
                continue
            fi

            push "$image"
            if [[ $? -ne 0 ]]; then
                continue
            fi

            success=true
            break
        done

        if [[ "${success}" == "false" ]]; then
            echo "[ERROR] Failed to manage image [${image}]"
            exit 1
        fi
    done <<< "${images}"

    echo "[SUCCESS] All images pushed to [${TO_REGISTRY}]"
}

# MAIN
while [[ $# -gt 0 ]]; do
key="$1"

case $key in
    --to )
        TO_REGISTRY=$2
        if [ -z "${TO_REGISTRY}" ]; then
            echo "[ERROR] Please provide a valid local registry location"
            exit 1
        fi
        shift # shift past argument
        shift # shift past value
    ;;
    --from )
        FROM_REGISTRY="${2:-${REGISTRY}}"
        shift # shift past argument
        shift # shift past value
    ;;
    --module )
        MODULE_NAME="${2}"
        shift # shift past argument
        shift # shift past value
    ;;
    --version )
        MODULE_VERSION="${2}"
        shift # shift past argument
        shift # shift past value
    ;;
    --local )
        USELOCAL=1
        shift # shift past argument
    ;;
    --module-dir )
        MODULE_DIRECTORY="${2}"
        shift # shift past argument
        shift # shift past value
    ;;
    -h | --help )
        usage
        exit
    ;;
    *) # unknown option
        echo "[ERROR] unknown argument: ${key}"
        usage
        exit 1
    ;;
esac
done

main
